/* 
 * cond.c
 *
 * condition variables broadcast (-DBROADCAST) and wait, pthreads
 *
 * Bernhard Rabe
 *
 * 18.05.2006
 */

#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>

static pthread_cond_t cond;
static pthread_mutex_t mutex;

/*pthread_cond_t cond = PTHREAD_COND_INITIALIZER;*/

void *thread1 (void *arg) {
   int ret, i;
   printf ("Thread 1 : %ld gestartet ...\n", pthread_self ());
   sleep (1);
    
   ret = pthread_mutex_lock (&mutex);
   if (ret != 0) 
   {
       fprintf(stderr,"pthread_mutex_lock:%s\n",strerror(ret));
       exit(EXIT_FAILURE);
   }    

   /*Kritischer Codeabschnitt */
   
   printf ("Thread 1 : %ld sendet das Signal fr die"
           " Condition Variable\n", pthread_self());
#ifdef BROADCAST   
   ret=pthread_cond_broadcast(&cond);
#else
   ret=pthread_cond_signal(&cond);
#endif   
   if (ret != 0) 
   {
       fprintf(stderr,"pthread_cond_signal/broadcast:%s\n",strerror(ret));
       exit(EXIT_FAILURE);
   }
   
   ret = pthread_mutex_unlock (&mutex);
   if (ret != 0) {
      printf ("Fehler bei unlock in Thread : %ld\n",
         pthread_self ());
      exit (EXIT_FAILURE);
   }
   printf ("Thread 1 : %ld beendet\n",pthread_self());
   pthread_exit ((void *) 0);
}

void *thread2 (void *arg) {
   int ret;
   
   printf ("Thread 2 : %ld wartet auf die Condition Variable\n",
            pthread_self ());
   if((ret=pthread_cond_wait (&cond, &mutex)))
   {
       fprintf(stderr,"pthread_cond_wait:%s\n",strerror(ret));
       exit(1);
   }
   printf ("Thread 2 : %ld gestartet ...\n", pthread_self ());
   printf ("Thread 2 : %ld beendet\n",pthread_self());
   if((ret=pthread_mutex_unlock(&mutex)))
   {
       fprintf(stderr,"pthread_mutex_unlock:%s\n",strerror(ret));
       exit(1);
   }
   pthread_exit ((void *) 0);
}

int main (void) {
   pthread_t th[4];
   int ret,i; 
 
   printf("%s version\n",
#ifdef BROADCAST
    "pthread_cond_broadcast"
#else
    "pthread_cond_wait"
#endif
   );    
    
   if((ret=pthread_cond_init (&cond, NULL)) ||
   (ret=pthread_mutex_init (&mutex, NULL)))
   {
          fprintf(stderr,"pthread_cond or mutex : %s\n",strerror(ret));
          exit(ret);
   }  
   
   if((ret=pthread_create (&th[0], NULL, thread1, NULL)))
   {
          fprintf(stderr,"pthread_create : %s\n",strerror(ret));
          exit(ret);
   }    
    
   for(i=1;i<4;i++) 
      if((ret=pthread_create (&th[i], NULL, thread2, NULL)))
      {
          fprintf(stderr,"pthread_create : %s\n",strerror(ret));
          exit(ret);
      }
      
   for(i=0;i<4;i++) 
     if((ret=pthread_join (th[i], NULL)))
         fprintf(stderr,"pthread_join : %s\n",strerror(ret));
   return EXIT_SUCCESS;
}
