/* pthread_mutex.c
 * 
 * pthreads / with mutex synchronisation compile with -DMUTEX
 * simple error handling
 *
 * Bernhard Rabe
 * 17.11.2004
 */

#include <pthread.h>
#include <stdio.h>
#include <unistd.h>

#define ERROR_AND_EXIT(msg,errval) {fprintf(stderr,"%s:%s\n",msg,strerror(errval)); exit(errval);}


/*global accessible*/
pthread_mutex_t mutex; 

/* Thread routine */
void* run(void* data)
{
    int id=*(int*)data,l,errval;
     
#ifdef MUTEX    
    if(errval=(0!=pthread_mutex_lock(&mutex))) ERROR_AND_EXIT("mutex_lock",errval)
#endif 
    
    for(l=0;l<10;l++)
    {
        printf("%c",id);
        usleep(10);
    }
    printf("\n");

#ifdef MUTEX    
    if(errval=(0!=pthread_mutex_unlock(&mutex))) ERROR_AND_EXIT("mutex_unlock",errval)
#endif    

    pthread_exit((void*)0);
}


int main(int argc, char *argv[])
{
    pthread_t thread[10];
    int errval;
    int ids[10]={'-','*','+','#','~','$','','%','^',''},i=2,l;
    
    if(argc>1)
    {
        i=atoi(argv[1]);
        if(i<2 || i>10)
        {
            printf("usage: %s [2-10]\n",argv[0]);
            exit(1);
        }
    }

#ifdef MUTEX    
    /*create mutex*/
    if(errval=(0!=pthread_mutex_init(&mutex,NULL)))
    {/*Fehler*/ 
        ERROR_AND_EXIT("mutex_init",errval)
    }
#endif    

    /*create and start threads*/    
    for(l=0;l<i;l++)
        if(errval=(0!=pthread_create(&thread[l],0,run,&ids[l])))
        {
            ERROR_AND_EXIT("pthread_create",errval)
        }
    printf("Waiting for Threads\n"); 
   
    /* wait for thread termination, must be in joinable mode */     
    for(l=0;l<i;l++)
        if(0!=pthread_join(thread[l],0))
        { 
            ERROR_AND_EXIT("pthread_join",errval)
        }
    
#ifdef MUTEX   
    /* free mutex */    
    if(0!=pthread_mutex_destroy(&mutex))
    { 
            ERROR_AND_EXIT("mutex_destroy",errval)
    }    
#endif    
    return 0;
}
