/*
 * event.c
 *
 * Manual reset (-DBROADCAST) and auto reset event, processes
 *
 * Bernhard Rabe
 *
 * 18.05.2006 
 */

#include <windows.h>
#include <stdio.h>

#define LOGMESSAGE(msg) {printf("%s %d\n",msg,GetCurrentProcessId());}

void printErrorAndExit(const char *msg, DWORD err)
{
	LPSTR lpMsgBuf;
	if(FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
			          FORMAT_MESSAGE_FROM_SYSTEM | 
					  FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						err,
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  (LPTSTR) &lpMsgBuf,
					  0,
					  NULL ))
	{
		fprintf(stderr,"%s : %s\n",msg,lpMsgBuf);
		LocalFree(lpMsgBuf);
	}
	else
	{
		fprintf(stderr,"Error at FormatMesage: %d\n",err=GetLastError());
	}
	exit(err);
}


int main(int argc, char *argv[])
{
    HANDLE hEvent;
    int c;

    if(argc<2)
    {
        fprintf(stderr,"usage: %s wait object name\n",argv[0]);
        return 1;        
    }

#ifdef BROADCAST
    printf("MANUAL REST EVENT\n");    
    hEvent=CreateEvent(NULL,TRUE,FALSE,argv[1]);
#else
    printf("AUTO RESET EVENT\n"); 
    hEvent=CreateEvent(NULL,FALSE,FALSE,argv[1]);
#endif    

    if(NULL==hEvent)
        printErrorAndExit("CreateEvent",GetLastError());
    
    if(ERROR_ALREADY_EXISTS==GetLastError())
    {
        LOGMESSAGE("Event already exists")
        LOGMESSAGE("Wait For Event")
        
        if(WAIT_FAILED==WaitForSingleObject(hEvent,INFINITE))
            printErrorAndExit("WaitForSingleObject",GetLastError());
        
        LOGMESSAGE("Event was signaled")
    }
    else
    {
        printf("Hit Enter to signal the event:");
        fgetc(stdin);
        if(!SetEvent(hEvent))
            printErrorAndExit("SetEvent",GetLastError());
    }
    
           
    if(!CloseHandle(hEvent))
        printErrorAndExit("CloseHandle",GetLastError());

    return 0;
}