/*
 *  mutex.c
 *  Threads and Mutex
 *
 *  Bernhard Rabe
 *  17.11.2004
 */
 
#include <windows.h>
#include <stdio.h>


HANDLE hMutex=NULL;

void printErrorAndExit(const char *msg, DWORD err,int id)
{
	LPSTR lpMsgBuf;
	if(FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
			          FORMAT_MESSAGE_FROM_SYSTEM | 
					  FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						err,
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  (LPTSTR) &lpMsgBuf,
					  0,
					  NULL ))
	{
		fprintf(stderr,"%s %d : %s\n",msg,id,lpMsgBuf);
		LocalFree(lpMsgBuf);
	}
	else
	{
		fprintf(stderr,"Error at FormatMesage: %d\n",err=GetLastError());
	}
	exit(err);
}


DWORD WINAPI Thread(LPVOID data)
{
    int id=*(int*)data,l;
#ifdef MUTEX    
    if(WAIT_OBJECT_0!=WaitForSingleObject(hMutex,INFINITE))
        printErrorAndExit("Error accessing the Mutex in",GetLastError(),id);
#endif    
    for(l=0;l<10;l++)
    {
        printf("%c",id);
        Sleep(1); /* Big computation here */
    }
    printf("\n");
#ifdef MUTEX    
    if(!ReleaseMutex(hMutex))
        printErrorAndExit("Error freeing the Mutex in",GetLastError(),id);
#endif    
    return 0;    
}


int main(int argc, char *argv[])
{
    HANDLE hThread[10];
    int ids[10]={'-','*','+','#','~','$','','%','^',''},i=2,l;
    if(argc>1)
    {
        i=atoi(argv[1]);
        if(i<2 || i> 10)
        {
            printf("Only 2-10 Threads allowed\n");
            printf("Usage: %s   - 2 Threads\nUsage: %s  [2-10] - 2-10 Threads\n");
            exit(1);
        }
    }
    
#ifdef MUTEX 
# ifdef MUTEX_OWNED    
    hMutex=CreateMutex(NULL,TRUE,NULL);
# else    
    hMutex=CreateMutex(NULL,FALSE,NULL);
# endif    
    if(INVALID_HANDLE_VALUE==hMutex)
        printErrorAndExit("Error creating the Mutex",GetLastError(),-1);
#endif
    
    for(l=0;l<i;l++)
    {       //CREATE_SUSPENDED
        DWORD id;
        if(NULL==(hThread[l]=CreateThread(NULL,0,Thread,&ids[l],0, &id)))
            printErrorAndExit("Error creating Thread",GetLastError(),-1);
    }
  
    printf("Waiting for Threads\n");
#ifdef MUTEX_OWNED
    if(!ReleaseMutex(hMutex))
        printErrorAndExit("ReleaseMutex",GetLastError(),-1);
#endif    
    if(WaitForMultipleObjects(i,hThread,TRUE,INFINITE)==WAIT_FAILED)
        printErrorAndExit("Error waiting for Threads",GetLastError(),-1);
        
    for(l=0;l<i;l++)
        if(!CloseHandle(hThread[l]))
            fprintf(stderr,"Error CloseHandle\n");
    
#ifdef MUTEX
    if(!CloseHandle(hMutex))
        printErrorAndExit("CloseHandle Mutex",GetLastError(),-1);
#endif    
    return 0;
}