/*
 * proc_mutex.c
 *
 * named mutex accross processes
 *
 * Bernhard Rabe
 *
 * 17.05.2006
 */

#include <windows.h>
#include <stdio.h>

#define LOGMESSAGE(msg) printf("%s %d\n",msg,GetCurrentProcessId());

void printErrorAndExit(const char *msg, DWORD err)
{
	LPSTR lpMsgBuf;
	if(FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
			          FORMAT_MESSAGE_FROM_SYSTEM | 
					  FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						err,
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  (LPTSTR) &lpMsgBuf,
					  0,
					  NULL ))
	{
		fprintf(stderr,"%s : %s\n",msg,lpMsgBuf);
		LocalFree(lpMsgBuf);
	}
	else
	{
		fprintf(stderr,"Error at FormatMesage: %d\n",err=GetLastError());
	}
	exit(err);
}



int main(int argc, char *argv[])
{
    HANDLE hMutex;
    int c;
    if(argc<2)
    {
        fprintf(stderr,"usage: %s wait object name\n",argv[0]);
        return 1;        
    }
    
    hMutex=CreateMutex(NULL,FALSE,argv[1]);
    if(NULL==hMutex)
        printErrorAndExit("CreateMutex",GetLastError());
    
    if(ERROR_ALREADY_EXISTS==GetLastError())
    {
        LOGMESSAGE("Mutex already exists")
    }
    
    do
    {
        LOGMESSAGE("Wait For Mutex")
        
        if(WAIT_FAILED==WaitForSingleObject(hMutex,INFINITE))
            printErrorAndExit("WaitForSingleObject",GetLastError());
        
        LOGMESSAGE("Enter Critical Section")
        
        printf("\nHit <Enter> to leave Critical Section: ");
        c=fgetc(stdin);
        LOGMESSAGE("Leave Critical Section")
        if(!ReleaseMutex(hMutex))
            printErrorAndExit("ReleaseMutex",GetLastError());
    }while(c!='q');
    
    if(!CloseHandle(hMutex))
        printErrorAndExit("CloseHandle",GetLastError());
    return 0;
}